{% extends "account/base.html" %}

{% load i18n %}
{% load account socialaccount %}

{% block head_title %}{% trans "Sign In" %}{% endblock %}

{% block content %}
  <div class="user-profile">
      {% if user.is_authenticated %}
      <h1>
          {% trans "Welcome" %}
          {{ user.username }}
      </h1>
      <p>
          <a href="{% url 'account_email' %}">
              {% trans "Change E-mail" %}
          </a>
          <a href="{% url 'account_logout' %}">
              {% trans "Logout" %}
          </a>
      </p>
      {% else %}
      <h1>
          {% blocktrans %}Welcome{% endblocktrans %}
      </h1>
          <a href="{% url 'account_login' %}">
              {% trans "Login" %}
          </a>
          <a href="{% url 'account_signup' %}">
              {% trans "Register" %}
          </a>
      {% endif %}
  </div>
{% endblock %}
